header
{
  import java.util.*;			 	
}
class PovrayLexer extends Lexer;


options
{
  filter=SEPARATEURS; // ne pas tenir compte des separateurs
  exportVocab=Pov;
  charVocabulary='\3'..'\377'; // pour pouvoir lire les accents
}
// protected indique les regles qui ne seront utilisees que dans le lexer.
protected DIGIT : '0' .. '9' ;
protected LETTER : 'a'..'z' | 'A'..'Z' | '_' | '\'';
protected SEPARATEURS : ' ' | '\t' | '\n'{newline();} | '\r' | COMMENT;
protected COMMENT : "//" (~'\n')* '\n' ;

protected DOT : '.';
protected PLUS : '+';
MOINS: '-';
DIV: '/' {LA(1) != '/'}?;
ADD: '+';
MUL: '*';
LPAR: '(';
RPAR: ')';

// Ici commencent les regles publiques
BEGBLOCK : '{';
ENDBLOCK : '}';
BEGVEC : '<';
ENDVEC : '>';
VIRG :  ',';

NAME : LETTER (LETTER|DIGIT)* ;
NUMBER: (((DIGIT)+ ((DOT) (DIGIT)+)?) | (DOT (DIGIT)+));
protected ABS :   ((DIGIT)+ ((DOT) (DIGIT)+)?) | (DOT (DIGIT)+);

class PovrayParser extends Parser ;
options
{
  exportVocab=Pov;
}

// Un fichier Povray est constitue de Camera, Box, Light, Plane, Cone,
// Cylindre, Sphere, Union
povfile
{ Camera c ; Box b ; Light l ; Plane p ; Cone co ; Cylindre cy ; Sphere s ; Union u ; DrawC fichier = new DrawC();}
:
(
 c = camera { c.pov2C(fichier);} |
 b = box { b.pov2C(fichier);} |
 l = light { l.pov2C(fichier);} |
 p = plane { p.pov2C(fichier);} |
 co = cone { co.pov2C(fichier);} |
 cy = cylindre { cy.pov2C(fichier);} |
 s = sphere { s.pov2C(fichier);} |
 u = union { u.pov2C(fichier);}
 )+{fichier.finDraw();fichier.remplirFichier("alldessiner.c");};


// Objects

// Union

union returns [ Union c=null ]
{ c= new Union () ;Forme o = null; Modif m = null;} :
"union" BEGBLOCK (o = forme {c.addObject(o);} )+ (m=modifier {c.addModif(m);})* ENDBLOCK
{
}
;

forme returns [Forme o=null]
{ Box b ;  Plane p ; Cone co ; Sphere s; Cylindre cy;Union u; Light l;} 
:
b = box{o = b;}  |
p = plane {o = p;}|
co = cone {o= co;}|
s = sphere { o=s;}|
cy = cylindre {o = cy;}|
u = union {o = u;}|
l = light {o = l;}
;

// Camera

// Une camera contient location et look_at comme vecteurs
camera returns [Camera c=null]
{ Point3D[] a=null;}
:
"camera" BEGBLOCK a=camattributs ENDBLOCK
{
  c = new Camera (a[0],a[1]);
}
;

camattributs returns [Point3D[] a=new Point3D[2]]
{Point3D b=null ; int i=0;}
: (b=camattribut{a[i++] = b;})+ ;

camattribut returns [Point3D t= null]
{Point3D a=null;}
:
a=location{t=a;} | a= look{t=a;} ;

location returns [Point3D a=null]
{ Point3D p =null; }
:
"location" p=vector3d {a = p; } ;

// Cylindre

cylindre returns [ Cylindre cy = null ]
{Point3D a=null, b=null; Modif o = null; cy = new Cylindre ();double c;}
: "cylinder" BEGBLOCK  a=vector3d {cy.setPointa(a);} (VIRG)? b=vector3d {cy.setPointb(b);} (VIRG)? c=chiffre {cy.setRayon(c);}
(o = modifier {cy.addModif(o);})+ ENDBLOCK
{   }
;

// Sphere

sphere returns [ Sphere s = null ]
{ Point3D a = null ;double b;s = new Sphere(); Modif m =null;}
: "sphere" BEGBLOCK a=vector3d {s.setCentre(a);} (VIRG)? b=chiffre{s.setRayon(b);}
(m = modifier { s.addModif(m);})+ ENDBLOCK
{ }
;

// Box

box returns [ Box c=null ] 
{ Point3D a=null, b=null; ; Modif o = null;c = new Box ();}
:       
"box" BEGBLOCK a=vector3d {c.setPointa(a);} (VIRG)? b=vector3d {c.setPointb(b);}(o = modifier { c.addModif(o); })+ ENDBLOCK
{
  
}
;

// Plane

plane returns [Plane c=null]
{Point3D axe=null; Couleur colors = null;}
:
"plane" BEGBLOCK axe = axis colors = pigment ENDBLOCK
{
  c = new Plane (axe, colors);
}
;

axis returns [Point3D axe = null]
{int a; double coor;}
:  ("x"{a = 0;}|"y"{a = 1;}|"z"{a = 2;}) (VIRG)? coor=chiffre
{
  
  
  axe = new Point3D();
  switch(a)
  {
  case 0 : axe.x =  coor;
    axe.y = 0;
    axe.z = 0;
    break;
  case 1 : axe.x = 0;
    axe.y =  coor;
    axe.z = 0;
    break;
  case 2:  axe.x = 0;
    axe.y = 0;
    axe.z = coor;
    break;
  }
} ;

// Cone

cone returns [ Cone co=null ]
{ Point3D a=null, c=null; Modif o = null; double b, d;co= new Cone();}
:      
"cone" BEGBLOCK a=vector3d {co.setBase(a);} (VIRG)? b=chiffre {co.setRayon1(b);}
(VIRG)? c=vector3d {co.setTop(c);}(VIRG)? d=chiffre {co.setRayon2(d);}
(o = modifier { co.addModif(o);})+ ENDBLOCK

;

// Light

light returns [Light l=null] 
{ Expr3D p; l = new Light(); }
: "light_source" BEGBLOCK p=expr3D {l.setPos(p);} {lightattributs(l);} ENDBLOCK { l.setNumero(Constants.nlights++);};

lightattributs [ Light l ] :
{Expr3D c; double t; Point3D co;}
( co=color {l.setCouleur (co);}| 
  spotlight {l.setTypeLight(1);} |
  t = radius  {l.setRadius(t);} |
  t= falloff  {l.setFalloff(t);} |
  t= tightness  {l.setTightness(t);} |
  c= point {l.setPoint (c);}
  )+
;

// Attributs d'objets


texture returns [Texture t = null]
{Couleur colors = null; }:
("texture" BEGBLOCK colors=pigment ENDBLOCK { t = colors;})
     | ( colors=pigment {t = colors;})
{  }
;

pigment returns [Couleur color = null] :
{ int i= 0;Point3D color1=null;Point3D color2=null;Point3D[] pig = new Point3D[2]; color = new Couleur();}
"pigment" BEGBLOCK (("checker" color1 = unpigment color2 = unpigment) | (color1=unpigment )) ENDBLOCK 
{pig[0] = color1;pig[1] = color2; color.setColor(pig);};
            
unpigment returns [ Point3D ret=null]:
{ Point3D p=null; }
"color" "rgb" p=vector3d {ret= p;};

color returns [ Point3D p = null ]:
"color" "rgb" p=vector3d;

spotlight : "spotlight" ;

radius returns [double a=0]: "radius" a=chiffre {};
falloff returns  [double a=0] : "falloff" a=chiffre {};
tightness returns  [double a=0] : {} "tightness" a=chiffre  ;
point  returns [ Expr3D p = null ]: 
"point_at" p=expr3D ;


look returns [ Point3D b=null ]
{Point3D p = null; }
:
"look_at" p=vector3d {b = p; } ;

// Transformations d'objets

modifier returns [Modif o=null]
:
(o=rotation)
|(o=texture)
|(o=translate)
|(o=scale)
;
     
rotation returns [Rotation r = null]:
{ Expr3D a=null ; }
"rotate" a=expr3D { r = new Rotation(a) ;} ;

scale returns [Scale sc = null]
{ Expr3D p; double d;}:
"scale" (p=expr3D { sc = new Scale (p) ;}
| d = chiffre 
{ sc = new Scale ( new Expr3D(""+d,""+d,""+d));  });

translate returns [Translation t = null]:
{ Expr3D p; }
"translate" p=expr3D { t = new Translation (p) ;} ;

// Expressions et nombres

vector3d returns [Point3D p = null]
{ int ia, ib, ic; double a, b, c;}
:       
BEGVEC a=chiffre VIRG b=chiffre VIRG c=chiffre ENDVEC 
{ 
  p = new Point3D (a, b, c);
}
;				

expr3D  returns [Expr3D p = null]:
{ String a, b, c;}
 
BEGVEC a=expression VIRG b=expression VIRG c=expression ENDVEC 
{ 
  p = new Expr3D (a, b, c);
}
;

chiffre returns [double i=0]:
PLUS a:NUMBER
{
  i =  Double.valueOf(a.getText()).doubleValue();
}
| MOINS b:NUMBER
{
  i = - Double.valueOf(b.getText()).doubleValue();
}
| c:NUMBER
{
  i = Double.valueOf(c.getText()).doubleValue();
}
;
							
expression returns [ String p = null ]:
{ String p1,p2,p3;}
p1 = mulExpression ((ADD { p3 = "+";}|MOINS { p3 = "-";}) p2 = mulExpression{ p1 += p3 + p2;})*
{p = p1;};

mulExpression returns [ String p = null ]:
{String op1, op2;}
(op1=divExpression (MUL op2=divExpression {op1 +="*"+op2;})*)
{p=op1;};

divExpression returns[ String p = null ]:
{String op1, op2;}
(op1=signeExpression (DIV op2=signeExpression{op1 +="/"+op2;})*)
{p=op1;};

signeExpression returns [String p = ""]:
{String op =null;}
op=atom
{p+=op;};

atom returns [ String p = null]:
{ String p1;double a;}
( (a=chiffre{p=a+"";} )| ("clock"{p="clock";}) | (LPAR p1=expression RPAR
{ p = "("+p1+")";}));				
